/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <QDrag>
#include <QGraphicsSceneMouseEvent>
#include <QList>
#include <QMimeData>
#include <QPixmap>
#include <QUrl>

#include <QDebug>
#include "defines.h"
#include "EquationPixmapItem.h"

EquationPixmapItem::EquationPixmapItem(QGraphicsItem * parent)
        : QGraphicsPixmapItem(parent)
{

}

void EquationPixmapItem::setPixmap(const QString &file)
{
    imageFile = file;

    QGraphicsPixmapItem::setPixmap( QPixmap(file) );

}

void EquationPixmapItem::mousePressEvent( QGraphicsSceneMouseEvent * event)
{
    if (event->button() == Qt::LeftButton)
        dragStartPosition = event->pos();

    qDebug() << "Equation: click " << dragStartPosition;

    setCursor(Qt::ClosedHandCursor);

}

void EquationPixmapItem::mouseMoveEvent( QGraphicsSceneMouseEvent * event)
{
    qDebug() << "Equation: drag begin...";

    if (!(event->buttons() & Qt::LeftButton))
        return;

    if (QLineF(event->screenPos(), event->buttonDownScreenPos(Qt::LeftButton)).length() < QApplication::startDragDistance()) {
        return;
    }

    qDebug() << "Equation: dragging ";

    QDrag *drag = new QDrag(event->widget() );
    QMimeData *mimeData = new QMimeData;

    QList<QUrl> urls;
    urls << imageFile;


    mimeData->setUrls(urls);
    mimeData->setImageData( QImage(imageFile) );
    mimeData->setText(QImage(imageFile).text(INFO_HEADER_LATEX) );

    drag->setMimeData(mimeData);
    drag->setPixmap( pixmap() );

    drag->setHotSpot(dragStartPosition.toPoint() );

    drag->exec(Qt::CopyAction | Qt::LinkAction);

}

void EquationPixmapItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    setCursor(Qt::OpenHandCursor);
}
