/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMap>
#include <QtGui/QMainWindow>
#include <QTimer>
#include <QTime>
#include "parser.h"

#include "workspace.h"

class QAction;
class QActionGroup;
class QGraphicsView;
class QGraphicsScene;
class QGraphicsPixmapItem;
class QCloseEvent;
class QMenu;
class QModelIndex;
class EquationPixmapItem;
class LatexEditor;
class WidgetToolMenu;
class QPushButton;
class QToolButton;

namespace Ui
{
    class MainWindowClass;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void updateEquation(); // updates tex, png, and scene
    void updateImage(); // only updates the image file DVI2PNG
    void updateDisplay(); // updates scene

    void resolutionChanged(const int res);
    void fgColorChanged(const QColor &color);
    void bgColorChanged(const QColor &color);
    void modeChanged(bool mode);

    void insertSymbol(QAction *action);

private slots:
    void newEquation();
    void open();
    bool save();
    bool saveAs();
    bool exportToDVI();
    bool exportToTeX();
    bool exportToPDF();
    bool exportToPS();
    bool exportToGIF();
    bool exportToMathML();
    bool exportToSVG();

    void editorChanged();
    void about();

protected:
    void setActions();
    void setMathPanel();
    void setMathPanelTabsActions();

    void setSignals();
    void setCurrentFile(const QString &fileName);

    bool wishToSave(); // if user wishes to save modifications
    void loadEquation(const QString &fileName);
    bool saveEquation(const QString &fileName);
    QString saveTo(const QString &filter);
    void writePNGMetaData(const QString &pngFile);

    void loadTexTemplate();

    void writeSettings();
    void readSettings();
    void setupParser();
    void closeEvent(QCloseEvent *);

   /*
        Adds ToolButtons with Actions conforming the file found by file name at eqImage
    to the widget
    */
    void addButtonToMathTabs(const QString &eqImage, QWidget *widget);
    void addButtonToMathTabs(const QStringList &list, QWidget *widget);

    QString strippedName(const QString &fullFileName);
private:
    /*  MainWindow widgets */
    Ui::MainWindowClass *ui;

    EquationPixmapItem *equation;
    QGraphicsScene *scene;
    QTimer timer;
    LatexEditor *editor;
    Parser parser;
    Workspace workspace;

    /* Mathbar */
    WidgetToolMenu *toolDeco;
    WidgetToolMenu *toolFunc;
    WidgetToolMenu *toolDelim;
    WidgetToolMenu *toolAccents;
    WidgetToolMenu *toolOps;
    WidgetToolMenu *toolRelations;
    WidgetToolMenu *toolMisc;
    WidgetToolMenu *toolSets;
    WidgetToolMenu *toolGreek;
    WidgetToolMenu *toolGreekBig;
    WidgetToolMenu *toolSums;
    WidgetToolMenu *toolIntegrals;
    WidgetToolMenu *toolProd;
    WidgetToolMenu *toolArrows;
    WidgetToolMenu *toolSpacing;

    // data
    QString tempFileName; // path to output temp file
    QString curFile; // current file
    QString texTemplateContent; // content of the tex template

    int resolution;
    bool isInline;
    bool isDirty; // wheater to update equation
    QString fgColor;
    QString bgColor;

    int nUpdates;

};

#endif // MAINWINDOW_H
